/*
 * Copyright (C) 2003 Mirko Boehm <mirko@hackerbuero.org>
 */

#include <kconfig.h>

#include "kvaio.h"

#include <qlabel.h>
#include <qcstring.h>
#include <qevent.h>
#include <qdatastream.h>
#include <kmainwindow.h>
#include <klocale.h>
#include <kdebug.h>
#include <kdialogbase.h>
#include <dcopclient.h>
#include <kapplication.h>
extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <getopt.h>
#include <errno.h>
#include <linux/types.h>
#include <linux/sonypi.h>
}

#define BUTTON_PRESSED	1
#define BUTTON_RELEASED	2

KVaio::KVaio(QObject *parent, const char* name)
    : QObject(parent, name),
      mSimulateMouseWheelEvents(true),
      mDisp(0)
{
    mDriver = new KVaioDriverInterface(this);

    if(!mDriver->connectToDriver())
    {
        delete mDriver; mDriver = 0L;
        kdDebug() << "KVaio: Cannot connect to driver." << endl;
    } else {
        kdDebug() << "KVaio: Connected to SonyPI driver." << endl;
        connect(mDriver, SIGNAL(vaioEvent(int)), SLOT(slotVaioEvent(int)));
    }

    mDisp = XOpenDisplay(NULL);
    if(!mDisp)
    {
        kdDebug() << "KVaio ctor: Failed to open display. Very strange." << endl;
    }

    if(!mClient.attach())
    {
       kdDebug() << "KVaio ctor: cannot attach to DCOP server." << endl;
    }

    KConfig config("kmilodrc");

    loadConfiguration(&config);
}

KVaio::~KVaio()
{
    kdDebug() << "KVaio dtor: shutting down." << endl;
    if(mDriver!=0)
    {
        mDriver->disconnectFromDriver();
    }
    if(mClient.isAttached())
    {
        mClient.detach();
    }
}

void KVaio::slotVaioEvent(int event)
{
    QString text;
    QTextStream stream(text, IO_WriteOnly);
    uint kbModifiers;

    // kdDebug() << "KVaio::slotVaioEvent: got Vaio event " << event << "."
    //           << endl;
    // get keyboard modifiers:

    kbModifiers = kapp->keyboardModifiers();

    switch(event)
    {
    case SONYPI_EVENT_JOGDIAL_DOWN:
        if((kbModifiers & KApplication::ControlModifier) && mCTRL_JogDial_Brightness)
        {
            modifyBrightness(-16);
        }
        if((kbModifiers & KApplication::Modifier1) && mALT_JogDial_Volume)
        {
            modifyVolume(-1);
        }
        if(mSimulateMouseWheelEvents && mDisp)
        {
            simulateButton(mDisp, 5, BUTTON_PRESSED | BUTTON_RELEASED);
        }
        break;
    case SONYPI_EVENT_JOGDIAL_UP:
        if((kbModifiers & KApplication::ControlModifier) && mCTRL_JogDial_Brightness)
        {
            modifyBrightness(16);
            break;
        }
        if((kbModifiers & KApplication::Modifier1) && mALT_JogDial_Volume)
        {
            modifyVolume(+1);
        }
        if(mSimulateMouseWheelEvents && mDisp)
        {
            simulateButton(mDisp, 4, BUTTON_PRESSED | BUTTON_RELEASED);
        }
        break;
    case SONYPI_EVENT_JOGDIAL_PRESSED:
        if(mJogDialPress_MMBClick && mDisp)
        {
            simulateButton(mDisp, 2, BUTTON_PRESSED);
        }
        break;
    case SONYPI_EVENT_JOGDIAL_RELEASED:
        if(mJogDialPress_MMBClick && mDisp)
        {
            simulateButton(mDisp, 2, BUTTON_RELEASED);
        }
        break;
//    case SONYPI_EVENT_FNKEY_F3:
//        break;
//    case SONYPI_EVENT_FNKEY_F5:
//        showBrightnessDialog();
//        break;
    default:
        stream << "Unhandled event: " << event;
        if(mReportUnknownEvents) showTextMsg(text);
        kdDebug() << "KVaio::slotVaioEvent: event not handled." << endl;
    }
}

void KVaio::simulateButton(Display *disp, int button, int mask)
{
    XTestGrabControl(disp, True);
    if (mask & BUTTON_PRESSED)
    {
        XTestFakeButtonEvent(disp, button, True, 0);
    }
    if (mask & BUTTON_RELEASED)
    {
        XTestFakeButtonEvent(disp, button, False, 0);
    }
    XSync(disp, False);
    XTestGrabControl(disp, False);
}

// void KVaio::showBrightnessDialog()
// {
//     static bool active = false;

//    if(active == false)
//    {
//        active = true;
//        KDialogBase dialog(0, "BrightNessDialog", true, "Set LCD Brightness",
//                           KDialogBase::Ok, KDialogBase::Ok, true);

//        BrightnessWidget widget(&dialog);
//        dialog.setMainWidget(&widget);
//        dialog.adjustSize();

//        dialog.resize((int)(dialog.width()*1.5),
//                      dialog.minimumSize().height());

//        widget.setBrightness(mDriver->brightness());

//        connect(&widget, SIGNAL(brightnessChanged(int)),
//                mDriver, SLOT(setBrightness(int)));

//        dialog.exec();

//        active = false;
//    } else {
//        kapp->beep();
//    }
// }

bool KVaio::showTextMsg(const QString& msg)
{
    if(isKMiloDAvailable())
    {
        QByteArray data, replyData;
        QDataStream arg(data, IO_WriteOnly);
        QCString replyType;

        arg << msg;
        if (!mClient.call("kded", "kmilod", "displayText(QString)",
                         data, replyType, replyData))
        {
            kdDebug() << "KVaio::showTextMsg: there was some error using DCOP." << endl;
            return false;
        } else {
            return true;
        }
    } else {
        return false;
    }
}

bool KVaio::showProgressMsg(const QString& msg, int value)
{
    if(isKMiloDAvailable())
    {
        QByteArray data, replyData;
        QDataStream arg(data, IO_WriteOnly);
        QCString replyType;

        arg << msg << value;
        if (!mClient.call("kded", "kmilod", "displayProgress(QString,int)",
                         data, replyType, replyData))
        {
            kdDebug() << "KVaio::showProgressMsg: there was some error using DCOP."
                      << endl;
            return false;
        } else {
            return true;
        }
    } else {
        return false;
    }
}

bool KVaio::modifyBrightness(int step)
{
    int brightness = mDriver->brightness();
    int orig = brightness;

    brightness = brightness + step;
    if(brightness>255)
    {
        brightness = 255;
    }

    if(brightness<0)
    {
        brightness = 0;
    }

    if(orig != brightness)
    {
        mDriver->setBrightness(brightness);
        showProgressMsg(i18n("Display Brightness:"),
                        (int)((100.0*brightness)/255) );
        return true;
    } else {
        kapp->beep();
        return false;
    }
}

bool KVaio::modifyVolume(int /* step */)
{
    // we increase the volume if step is gt zero, and decrease it if
    // step is lt zero:
    if(mClient.isAttached())
    {
    }
    return false;
}

bool KVaio::isKMiloDAvailable()
{
    if(mClient.isAttached())
    {
        // kdDebug() << "KVaio::showTextMsg: attached to DCOP server." << endl;
        if(mClient.isApplicationRegistered("kded"))
        {
            QCStringList objects;

            // kdDebug() << "KVaio::showTextMsg: kded is registered at dcop server."
            //           << endl;
            objects = mClient.remoteObjects("kded");
            if(objects.contains("kmilod"))
            {
                // kdDebug() << "KVaio::showTextMsg: kmilod is available at kded."
                //           << endl;
                return true;
            } else {
                kdDebug() << "KVaio::isKMiloDAvailable: "
                          << "kmilod is NOT available at kded." << endl;
                return false;
            }
        } else {
            kdDebug() << "KVaio::isKMiloDAvailable: "
                      << "kded is NOT registered at dcop server." << endl;
            return false;
        }
    } else {
        kdDebug() << "KVaio::isKMiloDAvailable: "
                  << "kded is NOT registered at dcop server." << endl;
        return false;
    }
}

void KVaio::loadConfiguration(KConfig *k)
{
    k->setGroup("KVaio");

    mCTRL_JogDial_Brightness = k->readBoolEntry("CTRL_JogDial_Brightness", true);
    mALT_JogDial_Volume = k->readBoolEntry("ALT_JogDial_Volume", true);
    mJogDialPress_MMBClick = k->readBoolEntry("JogDial_Press_MMBClick", true);
    mReportUnknownEvents = k->readBoolEntry("Report_Unknown_Events", true);

    kdDebug() << "KVaio::loadConfiguration: " << endl
              << "       mCTRL_JogDial_Brightness:  " << mCTRL_JogDial_Brightness << endl
              << "       mALT_JogDial_Volume:       " << mALT_JogDial_Volume << endl
              << "       mJogDialPress_MMBClick:    " << mJogDialPress_MMBClick << endl
              << "       mReportUnknownEvents:      " << mReportUnknownEvents << endl;
}

int KVaio::brightness()
{
    return mDriver->brightness();
}

const KVaioDriverInterface* KVaio::driver()
{
    return mDriver;
}

#include "kvaio.moc"
